
import processing.opengl.*;
import javax.media.opengl.*; 
import javax.media.opengl.glu.*;
//import javax.media.opengl.glut.*;
import com.sun.opengl.util.GLUT;

import com.sun.opengl.util.texture.*; 
import javax.vecmath.*;
import com.sun.opengl.util.BufferUtil;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;

float FOVY = 45f;
float NEAR = .1f;
float FAR = 1000f;

PGraphicsOpenGL pgl;
GL gl;
GLU glu;
GLUT glut = new GLUT();
float angle = 10f;

Camera cam;

int[] viewportBounds = new int[4];
TextureObj tex1 = null;
TextureObj tex2 = null;
TextureObj texBlur = null;
//diffuse light color variables

//light position variables
float lx = 3.0;
float ly = 1.0;
float lz = 2.0;
float lw = 0.0;
  
void setup() 
{
  size(300, 300, OPENGL); 

  frame.setResizable(true); 
  viewportBounds = new int[]
  { 
    0,0, width, height               
  };

  tex1 = new TextureObj(this, "pehr.jpg");
  tex2 = new TextureObj(this, "andres.jpg");
  texBlur = new TextureObj(this, "gauss.jpg");


  cam = new Camera();

  background(0);
}

void initProjection(GL gl, GLU glu)
{
  gl.glViewport(0, 0, width, height);  
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(FOVY, (float)width/(float)height, NEAR, FAR);
}

void initModelview(GL gl)
{
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
}

int programID = -1;
int waveVarID = -1;
boolean firstTime = true;

void draw()
{
  background(0); //clear screen

  //get OpenGL context
  pgl = (PGraphicsOpenGL) g; 
  gl = pgl.gl;
  glu = pgl.glu;

  //override Processing defaults to work with normal OpenGL matrices
  initProjection(gl, glu);
  initModelview(gl);

 gl.glEnable(gl.GL_DEPTH_TEST);
 gl.glDisable(gl.GL_BLEND);

 setLighting();
 
  //handle camera view
  cam.renderCam(gl, glu);

 
  if (firstTime == true)
  {

    //load in shader program
    programID = initShaders("perPixel.vert", "perPixel.frag");

    firstTime = false;
  }

  if (programID <= 0)
  {
    println("ERROR - couldn't load shader program!");
  }
  else
  { 
    //Bind GLSL Program to overwrite fixed functionality
    gl.glUseProgram(programID);

    gl.glRotatef(angle, 1.0, 0.0, 0.0); //rotate on the x axis
    gl.glRotatef(angle, 0.0, 1.0, 0.0); //rotate on the y axis
    gl.glRotatef(angle, 0.0, 0.0, 1.0); //rotate on the z axis
      
    float mShininess[] = new float[]{5};
    float DiffuseMaterial[] = new float[]{1.0, 0.0, 0.0}; 
    float AmbientMaterial[] = new float[]{0.0, 0.0, 0.0};
    float SpecularMaterial[] = new float[]{1.0, 1.0, 1.0}; 
    
    gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, DiffuseMaterial, 0);
    gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, AmbientMaterial, 0);
    gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, SpecularMaterial, 0);
    gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, mShininess, 0);
    
   // glut.glutSolidSphere(8, 20, 20);
    glut.glutSolidTeapot(4);

    angle += 0.09f;
    //Un-bind GLSL Program to return to fixed functionality
    gl.glUseProgram(0);
  }
}


void setLighting() 
{

  
  float DiffuseLight[] = new float[]{1.0, 1.0, 1.0}; 
  float AmbientLight[] = new float[]{0.2, 0.2, 0.2};
  float SpecularLight[] = new float[]{1.0, 1.0, 1.0};
    
  gl.glLightfv (gl.GL_LIGHT0, gl.GL_SPECULAR, SpecularLight, 0); //change the light accordingly
  gl.glLightfv (gl.GL_LIGHT0, gl.GL_DIFFUSE, DiffuseLight, 0); //change the light accordingly
  gl.glLightfv (gl.GL_LIGHT0, gl.GL_AMBIENT, AmbientLight, 0); //change the light accordingly
    
  float LightPosition[] = {
    lx, ly, lz, lw  }; //set the LightPosition to the specified values

  gl.glLightfv (gl.GL_LIGHT0, gl.GL_POSITION, LightPosition, 0); //change the light accordingly
}

void resetProcessingMatrices(GL gl, GLU glu)
{
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(60, width/(float)height, .1f, 1000f);
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
  glu.gluLookAt(width/2.0f, height/2.0f, (height/2.0f) / tan(PI*60.0f / 360.0f), width/2.0f, height/2.0f, 0, 0, 1, 0); 
}



void keyPressed()
{
  switch(key)
  {
  case 'a':
    cam.moveCamX(-cam.translateSpeed);
    break;
  case 'z':
    cam.moveCamX(cam.translateSpeed);
    break;
  case 's':
    cam.moveCamY(cam.translateSpeed);
    break;
  case 'x':
    cam.moveCamY(-cam.translateSpeed);
    break;
  case 'd':
    cam.moveCamZ(-cam.translateSpeed);
    break;
  case 'c':
    cam.moveCamZ(cam.translateSpeed);
    break;

  case 'g':
    cam.rotateCamY(cam.rotateSpeed);
    break;
  case 'b':
    cam.rotateCamY(-cam.rotateSpeed);
    break;
  case 'h':
    cam.rotateCamZ(-cam.rotateSpeed);
    break;
  case 'n':
    cam.rotateCamZ(cam.rotateSpeed);
    break;
  case 'j':
    cam.rotateCamX(-cam.rotateSpeed);
    break;
  case 'm':
    cam.rotateCamX(cam.rotateSpeed);
    break;

  case 'q':
    cam.resetCamera();
    break;

  case '1':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 0f, 180f, 0f, 0f, 0f, 0f));
    break;
  case '2':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 0f, 0f, 180f, 0f, 0f, 0f));
    break;
  case '3':  
    cam.animations.add(new Animation(cam, millis(), millis() + 2000, 180f, 0f, 0f, 0f, 0f, 0f));
    break;
  case '4':  
    cam.animations.add(new Animation(cam, millis(), millis() + 300, 0f, 0f, 0f, 0f, 0f, +5f));
    break;
  case '5':  
    cam.animations.add(new Animation(cam, millis(), millis() + 300, 0f, 0f, 0f, 0f, 0f, -5f));
    break;
  }
}

void mouseDragged() 
{
  float rate = .5f;
  cam.rotateCamY((mouseX-pmouseX) * rate);
}




















